# -*- coding:utf-8 -*-
import os, ctypes, getpass, subprocess, pwd
from security_module import SecurityModule

from utils import check_user, check_group, check_login, enable_user, disable_user, create_file, create_symbolic, unlink_symbolic, chage_spwd
from log import log_error, log_debug

KYSEC_U_SERIAL=1
LIGHTDM_DEFAULTS=0

DATA_DIR="/usr/share/security-switch/data"
POLKIT_DEST="/etc/polkit-1/localauthority/10-vendor.d"
POLKIT_SRC=DATA_DIR+"/polkit"
POLKIT_CONF="kysec.pkla"
POLKITADM_DEST="/etc/polkit-1/localauthority.conf.d"
POLKITADM_SRC=DATA_DIR+"/polkit"
POLKITADM_CONF="52-kylin-admin.conf"
LIGHTDM_DEST="/usr/share/lightdm/lightdm.conf.d"
LIGHTDM_SRC=DATA_DIR+"/lightdm"
LIGHTDM_CONF="99-enable-manual-login.conf"
LIGHTDM_CONF_DEFAULTS="99-enable-manual-login-defaults.conf"
LIGHTDM_CONF_NODEFAULTS="99-enable-manual-login-nodefaults.conf"

class Admin3Module(SecurityModule):
	status_desc={ "0": "关闭", "1": "启用" }
	def __init__(self, auto_reset = False):
		SecurityModule.__init__(self, "admin3")
		self.auto_resetpw = auto_reset

	@staticmethod
	def getstatus():
		try:
			kyseclib = ctypes.cdll.LoadLibrary("libkysec.so.0")
		except OSError as e:
			return 0	
		else:
			return kyseclib.kysec_get_func_status(4)

	@classmethod
	def check_system(self, next_state):
		if next_state:
			src_file = POLKIT_SRC+"/"+POLKIT_CONF
			log_debug("检查'%s'是否存在..."%src_file)
			if not os.path.exists(src_file):
				return (False, "Polkit配置文件 '%s' 不存在"%src_file)
			src_file = POLKITADM_SRC+"/"+POLKITADM_CONF
			log_debug("检查'%s'是否存在..."%src_file)
			if not os.path.exists(src_file):
				return (False, "Polkit admim配置文件 '%s' 不存在"%src_file)

			if KYSEC_U_SERIAL:
				if LIGHTDM_DEFAULTS:
					src_file=LIGHTDM_SRC+"/"+LIGHTDM_CONF_DEFAULTS
				else:
					src_file=LIGHTDM_SRC+"/"+LIGHTDM_CONF_NODEFAULTS
				log_debug("检查'%s'是否存在..."%src_file)
				if not os.path.exists(src_file):
					return (False, "Lightdm配置文件 '%s' 不存在"%src_file)

			log_debug("检查用户组secadm的GID是否为600...")
			result, error=check_group("secadm", 600)	
			if not result:
				return (False, error)

			log_debug("检查用户组auditadm的GID是否为700...")
			result, error=check_group("auditadm", 700)	
			if not result:
				return (False, error)

			log_debug("检查用户secadm是否合法...")
			result, error=check_user("secadm", 600, "/home/secadm", ["secadm"])
			if not result:
				return (False, error)

			log_debug("检查用户auditadm是否合法...")
			if KYSEC_U_SERIAL:
				result, error=check_user("auditadm", 700, "/home/auditadm", ["auditadm", "syslog", "adm"])
			else:
				result, error=check_user("auditadm", 700, "/home/auditadm", ["auditadm", "adm"])
			if not result:
				return (False, error)

		return (True, None)

	def do_enable(self):

		if not self.getstatus():
			try:
				kysec_scenelib = ctypes.cdll.LoadLibrary("libkysec_scene.so.0")
				kysec_scenelib.kysec_scene_set_module_status(4, 1)  # 3adm
			except Exception as e:
				return (False, "libkysec_scene 打开失败")

			log_debug("启用root账户...")
			os.system("passwd -u root > /dev/null")
			result, error=enable_user("root", auto_reset = self.auto_resetpw)
			if not result:
				return (False, "无法启用root账户: %s"%error)

			log_debug("启用secadm账户...")
			# os.system("setfacl -m u:secadm:rw /etc/kysec/db/whlist.db")
			# os.system("setfacl -m u:secadm:rw /etc/kysec/tmp/.status")
			result, error=enable_user("secadm", auto_reset = self.auto_resetpw)
			if not result:
				return (False, "无法启用secadm账户: %s"%error)

			log_debug("启用auditadm账户...")
			result, error=enable_user("auditadm", auto_reset = self.auto_resetpw)
			if not result:
				return (False, "无法启用auditadm账户: %s"%error)
		else:
			os.system("passwd -u root > /dev/null")
			log_debug("三权分立已启用，跳过密码重置过程")



		my_env = {"PATH":"/usr/sbin:/usr/bin:/sbin:/bin"}
		log_debug("设置secadm用户")
		try:
			subprocess.check_call(["usermod", "-U", "-s", "/bin/bash", "secadm"], env=my_env)
		except Exception as e:
			return (False, "设置secadm账户shell配置出错: %s"%str(e))

		log_debug("设置auditadm用户")
		try:
			subprocess.check_call(["usermod", "-U", "-s", "/bin/bash", "auditadm"], env=my_env)
		except Exception as e:
			return (False, "设置auditadm账户shell配置出错: %s"%str(e))

		chage_spwd("root", True)
		chage_spwd("secadm", True)
		chage_spwd("auditadm", True)
		try:
			log_debug("启用Polkit配置...")
			os.system("mkdir -p %s" % POLKIT_DEST)
			create_symbolic(POLKIT_SRC+"/"+POLKIT_CONF, POLKIT_DEST+"/"+POLKIT_CONF)
		except Exception as e:
			log_error("Failed to update polkit configuration: %s"%str(e))
			return (False, "更新polkit配置时出错: %s"%str(e))
		try:
			log_debug("启用Polkit admin配置...")
			create_symbolic(POLKITADM_SRC+"/"+POLKITADM_CONF, POLKITADM_DEST+"/"+POLKITADM_CONF)
		except Exception as e:
			log_error("Failed to update polkit admin configuration: %s"%str(e))
			return (False, "更新polkit admin配置时出错: %s"%str(e))

		if KYSEC_U_SERIAL:
			p = subprocess.Popen('ps -ax | grep /usr/bin/Xwayland | grep -v grep', stdout=subprocess.PIPE, shell=True)
			out = p.stdout.readlines()
			if not len(out):
				try:
					log_debug("启用Lightdm配置...")
					if LIGHTDM_DEFAULTS:
						create_symbolic(LIGHTDM_SRC+"/"+LIGHTDM_CONF_DEFAULTS, LIGHTDM_DEST+"/"+LIGHTDM_CONF)
					else:
						create_symbolic(LIGHTDM_SRC+"/"+LIGHTDM_CONF_NODEFAULTS, LIGHTDM_DEST+"/"+LIGHTDM_CONF)
				except Exception as e:
					log_error("Failed to update lightdm configuration: %s"%str(e))
					return (False, "更新Lightdm配置时出错: %s"%str(e))
			else:
				try:
					log_debug("禁用Lightdm配置...")
					unlink_symbolic(LIGHTDM_DEST+"/"+LIGHTDM_CONF)
				except Exception as e:
					log_error("Failed to update lightdm configuration: %s"%str(e))
					return (False, "更新Lightdm配置时出错: %s"%str(e))

		# log_debug("创建文件 /.exectl ...")
		# result, error = create_file("/.exectl")
		# if not result:
		# 	return (False, "无法创建重打标记文件")
		try:
			os.system("mkdir -p /etc/kysec/tmp/")
			with open("/etc/kysec/tmp/.3adm_status", "w") as f:
				f.write("1")
		except Exception as e:
			return False, "3adm_status文件写入失败 %s" % e

		return (True, None)

	@classmethod
	def do_disable(self):
		try:
			kysec_scenelib = ctypes.cdll.LoadLibrary("libkysec_scene.so.0")
			kysec_scenelib.kysec_scene_set_module_status(4, 0)  # 3adm
		except Exception as e:
			return (False, "libkysec_scene 打开失败")

		my_env = {"PATH":"/usr/sbin:/usr/bin:/sbin:/bin"}

		chage_spwd("root", False)
		os.system("passwd -l root > /dev/null")

		log_debug("禁用secadm用户")
		try:
			pwd.getpwnam('secadm')
		except KeyError:
			log_debug("secadm用户不存在")
		else:
			chage_spwd("secadm", False)
			try:
				subprocess.check_call(["usermod", "-L", "-s", "/sbin/nologin", "secadm"], env=my_env)
			except Exception as e:
				return (False, "禁用secadm账户配置出错: %s"%str(e))

		log_debug("禁用auditadm用户")
		try:
			pwd.getpwnam('auditadm')
		except KeyError:
			log_debug("auditadm用户不存在")
		else:
			chage_spwd("auditadm", False)
			try:
				subprocess.check_call(["usermod", "-L", "-s" "/sbin/nologin", "auditadm"], env=my_env)
			except Exception as e:
				return (False, "禁用auditadm账户配置出错: %s"%str(e))

		try:
			log_debug("禁用Polkit配置...")
			unlink_symbolic(POLKIT_DEST+"/"+POLKIT_CONF)
		except Exception as e:
			log_error("Failed to update polkit configuration: %s"%str(e))
			return (False, "更新Polkit配置时出错: %s\n"%str(e))
		try:
			log_debug("禁用Polkit admin配置...")
			unlink_symbolic(POLKITADM_DEST+"/"+POLKITADM_CONF)
		except Exception as e:
			log_error("Failed to update polkit admin configuration: %s"%str(e))
			return (False, "更新Polkit admin配置时出错: %s\n"%str(e))

		if KYSEC_U_SERIAL:
			try:
				log_debug("禁用Lightdm配置...")
				unlink_symbolic(LIGHTDM_DEST+"/"+LIGHTDM_CONF)
			except Exception as e:
				log_error("Failed to update lightdm configuration: %s"%str(e))
				return (False, "更新Lightdm配置时出错: %s"%str(e))

		try:
			os.system("mkdir -p /etc/kysec/tmp/")
			with open("/etc/kysec/tmp/.3adm_status", "w") as f:
				f.write("0")
		except Exception as e:
			return False, "3adm_status文件写入失败 %s" % e
		# if status != 0:
		# 	return (False, "无法创建3adm_status文件")

		return (True, None)
